import cv2
import numpy as np
import matplotlib.pyplot as plt
from collections import deque

# Setup video capture
cap = cv2.VideoCapture(1)

buffer_size = 200
positions = deque(maxlen=buffer_size)

lower_color = np.array([0, 50, 50])
upper_color = np.array([10, 255, 255])
px_per_cm = 30.0
initial_y = None

fps = 30  # Your camera FPS
time_per_frame = 1 / fps

# Setup matplotlib interactive plot
plt.ion()
fig, ax = plt.subplots()
line, = ax.plot([], [], 'b-')
ax.set_xlim(0, 3)  # Show only 3 seconds at a time initially
ax.set_ylim(-10, 10)  # Adjust Y-axis limits to expected motion in cm
ax.set_xlabel("Time (seconds)")
ax.set_ylabel("Distance from start (cm)")
ax.set_title("Wilberforce Pendulum Vertical Motion")
ax.grid(True)

velocity_buffer = deque(maxlen=5)  # smoothing velocity

while True:
    ret, frame = cap.read()
    if not ret:
        break

    frame = cv2.flip(frame, 1)
    hsv = cv2.cvtColor(frame, cv2.COLOR_BGR2HSV)

    mask = cv2.inRange(hsv, lower_color, upper_color)
    mask = cv2.erode(mask, None, iterations=2)
    mask = cv2.dilate(mask, None, iterations=2)

    cnts, _ = cv2.findContours(mask.copy(), cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
    if len(cnts) > 0:
        c = max(cnts, key=cv2.contourArea)
        ((x, y), radius) = cv2.minEnclosingCircle(c)

        if radius > 5:
            center = (int(x), int(y))
            cv2.circle(frame, center, int(radius), (0, 0, 255), 2)

            if initial_y is None:
                initial_y = y

            relative_y_cm = (y - initial_y) / px_per_cm
            positions.append(relative_y_cm)

            cv2.putText(frame, f"{relative_y_cm:.2f} cm", (10, 30),
                        cv2.FONT_HERSHEY_SIMPLEX, 1, (0, 255, 0), 2)

    cv2.imshow("Wilberforce Pendulum Tracker", frame)

    # Prepare time axis in seconds
    times = [i * time_per_frame for i in range(len(positions))]
    neg_positions = np.negative(positions)

    # Only show the last 3 seconds
    visible_time = 3  # seconds
    visible_frames = int(fps * visible_time)

    times_to_plot = times[-visible_frames:]
    positions_to_plot = neg_positions[-visible_frames:]

    line.set_xdata(times_to_plot)
    line.set_ydata(positions_to_plot)

    if times_to_plot:
        ax.set_xlim(times_to_plot[0], times_to_plot[-1])
    if positions_to_plot:
        ymin = min(positions_to_plot) - 1
        ymax = max(positions_to_plot) + 1
        ax.set_ylim(ymin, ymax)

    fig.canvas.draw()
    fig.canvas.flush_events()

    # Calculate velocity (cm/frame) and convert to cm/s
    if len(positions) >= 2:
        velocity_per_frame = positions[-1] - positions[-2]
        velocity_cm_per_s = velocity_per_frame * fps
    else:
        velocity_cm_per_s = 0

    velocity_buffer.append(velocity_cm_per_s)
    smoothed_velocity = sum(velocity_buffer) / len(velocity_buffer)

    # Create black image for velocity display
    vel_display = np.zeros((100, 400, 3), dtype=np.uint8)
    cv2.putText(vel_display, f"Velocity: {smoothed_velocity:.2f} cm/s",
                (10, 60), cv2.FONT_HERSHEY_SIMPLEX, 0.8, (0, 255, 0), 2)
    cv2.imshow("Velocity", vel_display)

    if cv2.waitKey(1) & 0xFF == ord('q'):
        break

cap.release()
cv2.destroyAllWindows()
plt.ioff()
plt.show()
